from implugin.jinja2.widget import MultiWidget


class FormWidget(MultiWidget):

    @classmethod
    def from_form(cls, request, *args, **kwargs):
        form = cls.form(request, *args, **kwargs)
        widget = cls(form)
        return form, widget

    class Templates(object):
        field_error = 'implugin.formskit:templates/field_error.jinja2'
        form_error = 'implugin.formskit:templates/form_error.jinja2'
        begin = 'implugin.formskit:templates/begin.jinja2'
        end = 'implugin.formskit:templates/end.jinja2'
        text = 'implugin.formskit:templates/text.jinja2'
        password = 'implugin.formskit:templates/password.jinja2'
        select = 'implugin.formskit:templates/select.jinja2'
        hidden = 'implugin.formskit:templates/hidden.jinja2'
        submit = 'implugin.formskit:templates/submit.jinja2'

    def __init__(self, form):
        super().__init__()
        self.form = form

    def get_tag_id(self, name):
        return '%s_%s' % (self.form.get_name(), name)

    def begin(self, tagid=None, style=None, htmlcls=None):
        data = {}
        data['action'] = getattr(self.form, 'action', None)
        data['id'] = tagid
        data['name'] = self.form.get_name()
        data['style'] = style
        data['class'] = htmlcls
        return self.render_for(self.Templates.begin, data)

    def end(self):
        return self.render_for(self.Templates.end, {})

    def text(self, name, disabled=False, autofocus=False):
        return self._input('text', name, disabled, autofocus)

    def password(self, name, disabled=False, autofocus=False):
        return self._input('password', name, disabled, autofocus)

    def select(self, name, disabled=False, autofocus=False):
        return self._input('select', name, disabled, autofocus)

    def _base_input(self, name):
        data = {}
        data['name'] = self.form.fields[name].get_name()
        data['value'] = self.form.get_value(name, default='')
        data['values'] = self.form.get_values(name)
        data['field'] = self.form.fields[name]
        data['templates'] = self.Templates
        data['type'] = type
        data['str'] = str
        return data

    def _input(
        self,
        input_type,
        name,
        disabled=False,
        autofocus=False,
        **kwargs
    ):
        data = self._base_input(name)
        field = data['field']

        data['id'] = self.get_tag_id(name)
        data['label'] = field.label
        data['error'] = field.error
        data['messages'] = field.get_error_messages()
        data['value_messages'] = field.get_value_errors(default=[])
        data['disabled'] = disabled
        data['autofocus'] = autofocus
        data.update(kwargs)
        template = getattr(self.Templates, input_type)
        return self.render_for(template, data)

    def hidden(self, name):
        data = self._base_input(name)
        return self.render_for(self.Templates.hidden, data)

    def csrf_token(self):
        return self.hidden('csrf_token')

    def submit(self, label='', cls='btn-success', base_cls='btn btn-lg'):
        return self.render_for(
            self.Templates.submit,
            {
                'label': label,
                'class': cls,
                'base_class': base_cls
            }
        )

    def form_error(self):
        data = {}
        data['error'] = True if self.form.success is False else False
        data['messages'] = self.form.get_error_messages()
        return self.render_for(self.Templates.form_error, data)

    def __call__(self, *args, **kwargs):
        data = {}
        data['args'] = args
        data['kwargs'] = kwargs
        data['widget'] = self
        return self.render_for(self.template, data)



from mock import MagicMock

from formskit import Form

from impaf.testing import RequestCase
from impaf.testing import PyTestCase
from impaf.testing import cache

from ..widget import FormWidget


class TestFormWidget(RequestCase, PyTestCase):

    _object_cls = FormWidget

    @cache
    def mform(self):
        return MagicMock()

    @cache
    def object(self):
        return super().object(self.mform())

    @cache
    def mrender_for(self):
        return self.pobject(self.object(), 'render_for')

    def test_get_tag_id(self):
        form = self.mform()
        form.get_name.return_value = 'myname'

        assert self.object().get_tag_id('tagname') == 'myname_tagname'

    def test_begin(self):
        form = self.mform()
        render_for = self.mrender_for()
        obj = self.object()

        obj.begin('tagid', 'mystyle')

        render_for.assert_called_once_with(
            'implugin.formskit:templates/begin.jinja2',
            {
                'action': form.action,
                'id': 'tagid',
                'name': form.get_name.return_value,
                'style': 'mystyle',
            }
        )

    def test_end(self):
        render_for = self.mrender_for()
        obj = self.object()

        obj.end()

        render_for.assert_called_once_with(
            'implugin.formskit:templates/end.jinja2',
            {
            }
        )

    def test_text(self):
        name = 'myname'
        field = MagicMock()
        form = self.mform()
        form.fields = {
            name: field,
        }
        render_for = self.mrender_for()
        obj = self.object()

        obj.text(name, disabled='disabled-val', autofocus='autofocus-val')

        field.get_name.assert_called_once_with()
        field.get_error_messages.assert_called_once_with()
        field.get_value_errors.assert_called_once_with(default=[])
        form.get_value.assert_called_once_with(name, default='')
        form.get_values.assert_called_once_with(name)

        render_for.assert_called_once_with(
            'implugin.formskit:templates/text.jinja2',
            {
                'name': field.get_name.return_value,
                'value': form.get_value.return_value,
                'values': form.get_values.return_value,
                'field': field,
                'templates': obj.templates,
                'id': obj.get_tag_id(name),
                'label': field.label,
                'error': field.error,
                'messages': field.get_error_messages.return_value,
                'value_messages': field.get_value_errors.return_value,
                'disabled': 'disabled-val',
                'autofocus': 'autofocus-val',
            },
            prefix=None,
        )

    def test_password(self):
        name = 'myname'
        field = MagicMock()
        form = self.mform()
        form.fields = {
            name: field,
        }
        render_for = self.mrender_for()
        obj = self.object()

        obj.password(name, disabled='disabled-val', autofocus='autofocus-val')

        field.get_name.assert_called_once_with()
        field.get_error_messages.assert_called_once_with()
        field.get_value_errors.assert_called_once_with(default=[])
        form.get_value.assert_called_once_with(name, default='')
        form.get_values.assert_called_once_with(name)

        render_for.assert_called_once_with(
            'implugin.formskit:templates/password.jinja2',
            {
                'name': field.get_name.return_value,
                'value': form.get_value.return_value,
                'values': form.get_values.return_value,
                'field': field,
                'templates': obj.templates,
                'id': obj.get_tag_id(name),
                'label': field.label,
                'error': field.error,
                'messages': field.get_error_messages.return_value,
                'value_messages': field.get_value_errors.return_value,
                'disabled': 'disabled-val',
                'autofocus': 'autofocus-val',
            },
            prefix=None,
        )

    def test_select(self):
        name = 'myname'
        field = MagicMock()
        form = self.mform()
        form.fields = {
            name: field,
        }
        render_for = self.mrender_for()
        obj = self.object()

        obj.select(name, disabled='disabled-val', autofocus='autofocus-val')

        field.get_name.assert_called_once_with()
        field.get_error_messages.assert_called_once_with()
        field.get_value_errors.assert_called_once_with(default=[])
        form.get_value.assert_called_once_with(name, default='')
        form.get_values.assert_called_once_with(name)

        render_for.assert_called_once_with(
            'implugin.formskit:templates/select.jinja2',
            {
                'name': field.get_name.return_value,
                'value': form.get_value.return_value,
                'values': form.get_values.return_value,
                'field': field,
                'templates': obj.templates,
                'id': obj.get_tag_id(name),
                'label': field.label,
                'error': field.error,
                'messages': field.get_error_messages.return_value,
                'value_messages': field.get_value_errors.return_value,
                'disabled': 'disabled-val',
                'autofocus': 'autofocus-val',
            },
            prefix=None,
        )

    def test_hidden(self):
        name = 'myname'
        field = MagicMock()
        form = self.mform()
        form.fields = {
            name: field,
        }
        render_for = self.mrender_for()
        obj = self.object()

        obj.hidden(name)

        field.get_name.assert_called_once_with()
        form.get_value.assert_called_once_with(name, default='')
        form.get_values.assert_called_once_with(name)

        render_for.assert_called_once_with(
            'implugin.formskit:templates/hidden.jinja2',
            {
                'name': field.get_name.return_value,
                'value': form.get_value.return_value,
                'values': form.get_values.return_value,
                'field': field,
                'templates': obj.templates,
            },
        )

    def test_csrf_token(self):
        name = 'csrf_token'
        field = MagicMock()
        form = self.mform()
        form.fields = {
            name: field,
        }
        render_for = self.mrender_for()
        obj = self.object()

        obj.csrf_token()

        field.get_name.assert_called_once_with()
        form.get_value.assert_called_once_with(name, default='')
        form.get_values.assert_called_once_with(name)

        render_for.assert_called_once_with(
            'implugin.formskit:templates/hidden.jinja2',
            {
                'name': field.get_name.return_value,
                'value': form.get_value.return_value,
                'values': form.get_values.return_value,
                'field': field,
                'templates': obj.templates,
            },
        )

    def test_submit(self):
        render_for = self.mrender_for()
        obj = self.object()

        obj.submit('mylabel', 'cls', 'base-cls')

        render_for.assert_called_once_with(
            'implugin.formskit:templates/submit.jinja2',
            {
                'label': 'mylabel',
                'class': 'cls',
                'base_class': 'base-cls',
            },
        )

    def test_form_error(self):
        form = self.mform()
        form.success = True
        render_for = self.mrender_for()
        obj = self.object()

        obj.form_error()

        form.get_error_messages.assert_called_once_with()

        render_for.assert_called_once_with(
            'implugin.formskit:templates/form_error.jinja2',
            {
                'error': False,
                'messages': form.get_error_messages.return_value,
            },
        )



from pyramid.session import check_csrf_token

from implugin.beaker import BeakerRequestable

from formskit import Form
from formskit.formvalidators import FormValidator


class PostForm(Form, BeakerRequestable):

    def __init__(self, request):
        self.feed_request(request)
        super().__init__()

        self.add_form_validator(CsrfMustMatch())

    def reset(self):
        super().reset()
        self.init_csrf()

    def init_csrf(self):
        self.add_field('csrf_token')
        self.set_value('csrf_token', self.session.get_csrf_token())

    def validate(self):
        return super().validate(self.POST.dict_of_lists())


class CsrfMustMatch(FormValidator):

    message = "CSRF token do not match!"

    def validate(self):
        self.form.POST['csrf_token'] = self.form.get_value('csrf_token')
        return check_csrf_token(self.form.request, raises=False)



from mock import patch

from pytest import fixture
from pytest import yield_fixture

from impaf.testing import ControllerCase
from impaf.testing import ControllerFixture
from impaf.testing import RequestFixture
from impaf.testing import cache


class FormskitControllerFixture(ControllerFixture):

    @yield_fixture
    def madd_form(self, testable):
        patcher = patch.object(testable, 'add_form')
        with patcher as mock:
            yield mock

    @fixture
    def fform(self, madd_form):
        return madd_form.return_value


class FormFixture(RequestFixture):

    @fixture
    def testable(self, mrequest, registry):
        return self._testable_cls(mrequest)


class FormskitControllerCase(ControllerCase):

    @cache
    def madd_form(self):
        return self.pobject(self.object(), 'add_form')

    @cache
    def madd_form_widget(self):
        return self.pobject(self.object(), 'add_form_widget')

    @cache
    def mform(self):
        return self.madd_form().return_value

    @cache
    def mform_widget(self):
        return self.madd_form_widget().return_value



from mock import MagicMock

from formskit import Form

from impaf.testing import RequestCase
from impaf.testing import PyTestCase
from impaf.testing import cache

from ..models import PostForm
from ..models import CsrfMustMatch


class MockedPostForm(Form):

    @property
    @cache('instance')
    def reset(self):
        return MagicMock()

    @property
    @cache('instance')
    def validate(self):
        return MagicMock()


class ExamplePostForm(PostForm, MockedPostForm):

    def _get_request_cls(self):
        return lambda x: x


class TestFlashMessageController(RequestCase, PyTestCase):

    _object_cls = ExamplePostForm

    @cache
    def object(self):
        return super().object(self.mrequest())

    @cache
    def madd_form_validator(self):
        return self.pobject(PostForm, 'add_form_validator')

    @cache
    def minit_csrf(self):
        return self.pobject(PostForm, 'init_csrf')

    @cache
    def mCsrfMustMatch(self):
        return self.patch('implugin.formskit.models.CsrfMustMatch')

    def test_init(self):
        add_form_validator = self.madd_form_validator()
        init_csrf = self.minit_csrf()
        CsrfMustMatch = self.mCsrfMustMatch()

        obj = self.object()

        add_form_validator.assert_called_once_with(CsrfMustMatch.return_value)
        init_csrf.assert_called_once_with()

    def test_reset(self):
        self.mregistry()
        obj = self.object()
        init_csrf = self.minit_csrf()
        super(PostForm, obj).reset.reset_mock()

        obj.reset()

        super(PostForm, obj).reset.assert_called_once_with()

        init_csrf.assert_called_once_with()

    def test_validate(self):
        post = self.mPOST()
        obj = self.object()

        result = obj.validate()

        assert result == super(PostForm, obj).validate.return_value
        post.dict_of_lists.assert_called_once_with
        super(PostForm, obj).validate.assert_called_once_with(
            post.dict_of_lists.return_value
        )


class TestCsrfMustMatch(RequestCase, PyTestCase):

    _object_cls = CsrfMustMatch

    @cache
    def mform(self):
        obj = MagicMock()
        obj.POST = self.mPOST()
        return obj

    @cache
    def object(self):
        obj = super().object()
        obj.set_form(self.mform())
        return obj

    @cache
    def mcheck_csrf_token(self):
        return self.patch('implugin.formskit.models.check_csrf_token')

    def test_validate(self):
        post = self.mPOST()
        form = self.mform()
        check_csrf_token = self.mcheck_csrf_token()
        obj = self.object()

        assert obj.validate() == check_csrf_token.return_value

        assert post['csrf_token'] == form.get_value.return_value
        form.get_value.assert_called_once_with('csrf_token')
        check_csrf_token.assert_called_once_with(form.request, raises=False)



from impaf.controller import Controller

from .widget import FormWidget


class FormskitController(Controller):

    def add_form(
        self,
        formcls,
        name='form',
        widgetcls=FormWidget,
        *args,
        **kwargs
    ):
        form = formcls(self.request, *args, **kwargs)
        widget = widgetcls(form)
        self.add_widget(name, widget)
        return form

    def add_form_widget(self, widget, name='form', *args, **kwargs):
        form, widget = widget.from_form(self.request, *args, **kwargs)
        self.add_widget(name, widget)
        return form



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
    'impaf',
    'formskit',
    'impaf-beaker',
]

if __name__ == '__main__':
    setup(
        name='impaf-formskit',
        version='0.1.3',
        description='Formskit plugin for Impaf.',
        license='Apache License 2.0',
        packages=find_packages('src'),
        package_dir={'': 'src'},
        namespace_packages=['implugin'],
        install_requires=install_requires,
        include_package_data=True,
        zip_safe=False,
    )



from mock import MagicMock

from impaf.testing import ControllerCase
from impaf.testing import PyTestCase

from ..controller import FormskitController


class TestFlashMessageController(ControllerCase, PyTestCase):

    _object_cls = FormskitController

    def test_add_form(self):
        formcls = MagicMock()
        widgetcls = MagicMock()
        madd_widget = self.madd_widget()

        form = self.object().add_form(
            formcls,
            'myname',
            widgetcls,
            'arg',
            kw='arg',
        )

        formcls.assert_called_once_with(self.mrequest(), 'arg', kw='arg')
        form = formcls.return_value
        widgetcls.assert_called_once_with(form)
        madd_widget.assert_called_once_with('myname', widgetcls.return_value)
